<?php
namespace Ig\IgRuckzuckevent\Controller;

use Ig\IgRuckzuckevent\Domain\Model\Eventgroup;
use Ig\IgRuckzuckevent\Domain\Model\FileReference;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Utility\DebuggerUtility;
use Ig\IgRuckzuckevent\Utility\UserUtility;
use Internetgalerie\IgDynval\Controller\DynamicValidationActionController;
use Psr\Http\Message\ResponseInterface;

/**
 * The CalendarController handles all actions directly related to Events.
 */
class CalendarController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController
{
    use DynamicValidationActionController;

    /**
     * The repository for events
     *
     * @var \Ig\IgRuckzuckevent\Domain\Repository\EventRepository
     */
    protected $eventRepository = null;

    /**
     * The repository for events
     *
     * @var \Ig\IgRuckzuckevent\Domain\Repository\EventgroupRepository
     */
    protected $eventgroupRepository = null;

    /**
     * feUserRepository
     *
     * @var \Ig\IgRuckzuckevent\Domain\Repository\FeUserRepository
     */
    protected $feUserRepository = null;

    /**
     * feUserGroupRepository
     *
     * @var \Internetgalerie\IgFrontendUser\Domain\Repository\FrontendUserGroupRepository
     */
    protected $feUserGroupRepository = null;

    /**
     * @param \Ig\IgRuckzuckevent\Domain\Repository\EventRepository $eventRepository
     */
    public function injectEventRepository(\Ig\IgRuckzuckevent\Domain\Repository\EventRepository $eventRepository): void
    {
        $this->eventRepository = $eventRepository;
    }

    /**
     * @param \Ig\IgRuckzuckevent\Domain\Repository\EventgroupRepository $eventgroupRepository
     */
    public function injectEventgroupRepository(\Ig\IgRuckzuckevent\Domain\Repository\EventgroupRepository $eventgroupRepository): void
    {
        $this->eventgroupRepository = $eventgroupRepository;
    }

    /**
     * @param \Ig\IgRuckzuckevent\Domain\Repository\FeUserRepository $feUserRepository
     */
    public function injectFeUserRepository(\Ig\IgRuckzuckevent\Domain\Repository\FeUserRepository $feUserRepository): void
    {
        $this->feUserRepository = $feUserRepository;
    }

    /**
     * @param \Internetgalerie\IgFrontendUser\Domain\Repository\FrontendUserGroupRepository $feUserGroupRepository
     */
    public function injectFeUserGroupRepository(\Internetgalerie\IgFrontendUser\Domain\Repository\FrontendUserGroupRepository $feUserGroupRepository): void
    {
        $this->feUserGroupRepository = $feUserGroupRepository;
    }


    /**
     * Initialize the current action
     *
     * @return void
     */
    public function initializeAction(): void
    {
        if (isset($this->arguments['event']) || isset($this->arguments['newEvent'])) {
            if (isset($this->arguments['event'])) {
                $event = $this->arguments['event'];
            } else {
                $event = $this->arguments['newEvent'];
            }

            // Set date format of date fields
            foreach (['dateFrom', 'dateTo', 'dateDeadline'] as $dateField) {
                $event->getPropertyMappingConfiguration()
                    ->forProperty($dateField)
                    ->setTypeConverterOption(
                        'TYPO3\\CMS\\Extbase\\Property\\TypeConverter\\DateTimeConverter',
                        \TYPO3\CMS\Extbase\Property\TypeConverter\DateTimeConverter::CONFIGURATION_DATE_FORMAT,
                        'd.m.Y'
                    );
            }

            // Set time format of time fields
            /*foreach (['timeFrom', 'timeTo', 'timeDeadline'] as $timeField) {
              $event->getPropertyMappingConfiguration()
              ->forProperty($timeField)
              ->setTypeConverter(GeneralUtility::makeInstance('Ig\\IgRuckzuckevent\\Property\\TypeConverter\\TimestampConverter'));
              }*/
        }
    }

    /**
     * List all events with possibility to sort and filter them
     *
     * @return void
     */
    public function listAction(): ResponseInterface
    {
        $eventgroups = $this->eventgroupRepository->findAll();
        $eventsPerDayAndTime = [];

        if ($this->request->hasArgument('message')) {
            $success = $this->request->getArgument('message');

            if ($success == 'success') {
                $message = 'Sie haben sich erfolgreich an einem Kurs angemeldet. Prüfen Sie bitte Ihr E-Mail-Konto.';
            }
        }
        /*		echo('a');
                $e=$this->eventRepository->findByUid(2434); // error bei nested level 1000 /etc/php/7.0/mods-available/xdebug.ini
                $e=$this->eventRepository->findByUid(8024); // geht
                echo('b');
                exit(0);
        */

        $dDate=new \DateTime('2020-05-03');
        for ($d=1;$d<=7;$d++) {
            $eventsPerDayAndTime[$d]= [
                'entries' => [],
                'day' => [
                    'nr' => $d,
                    'text' =>  $dDate->format('Y-m-d')
                    ]
            ];
        }
        foreach ($eventgroups as $eventgroup) {
            /*
              if($eventgroup->getUid() == 18  && 0) {
              //$event = $eventgroup->getCurrentEvent();
              continue;

              }
            */
            $event = $eventgroup->getCurrentEvent();
            if ($event) {
                if (!is_array($eventsPerDayAndTime[$event->getIsoDay()][$event->getDaytime()])) {
                    $eventsPerDayAndTime[$event->getIsoDay()][$event->getDaytime()] = [];
                }
                // ist event schon vorhanden?
                $eventSchonVorhanden=false;
                foreach ($eventsPerDayAndTime[$event->getIsoDay()][$event->getDaytime()] as $eNr=>$e) {
                    if ($e->getTitle()==$event->getTitle() && $e->getSubTitle()==$event->getSubTitle() && $e->getTimeFrom()===$event->getTimeFrom() && $e->getTimeTo()===$event->getTimeTo()) {
                        $eventSchonVorhanden=true;
                        break;
                    }
                }
                if (!$eventSchonVorhanden) {
                    $eventsPerDayAndTime[$event->getIsoDay()][$event->getDaytime()][] = $event;
                }
                usort($eventsPerDayAndTime[$event->getIsoDay()][$event->getDaytime()], function ($a, $b) {
                    return strtotime($a->getTimeFrom()) > strtotime($b->getTimeFrom());
                });
            }
        }

        $morningMax = 1;
        $afternoonMax = 1;
        $eveMax = 1;
        foreach ($eventsPerDayAndTime as &$eventsPerTime) {
            $morningMax = max($morningMax, empty($eventsPerTime['morning']) ? 0 : count($eventsPerTime['morning']));
            $afternoonMax = max($afternoonMax, empty($eventsPerTime['afternoon']) ? 0 : count($eventsPerTime['afternoon']));
            $eveMax = max($eveMax, empty($eventsPerTime['eve']) ? 0 : count($eventsPerTime['eve']));
        }
        for ($d=1;$d<=7;$d++) {
            if (empty($eventsPerDayAndTime[$d]['morning']) && empty($eventsPerDayAndTime[$d]['afternoon']) && empty($eventsPerDayAndTime[$d]['eve'])) {
                unset($eventsPerDayAndTime[$d]);
            }
        }
    foreach ($eventsPerDayAndTime as &$day) {
            if (is_array($day)) {
                $startDayMorning = empty($day['morning']) ? 0 : count($day['morning']);
                for ($i = $startDayMorning; $i < $morningMax; $i++) {
                    $day['morning'][] = 'empty';
                }
                $startDayAfternoon = empty($day['afternoon']) ? 0 : count($day['afternoon']);
                for ($i = $startDayAfternoon; $i < $afternoonMax; $i++) {
                    $day['afternoon'][] = 'empty';
                }
                $startDayEve = empty($day['eve']) ? 0 : count($day['eve']);
                for ($i = $startDayEve; $i < $eveMax; $i++) {
                    $day['eve'][] = 'empty';
                }
            }
        }

        if ($this->request->hasArgument('managementUser') && UserUtility::isAdminLoggedIn()) {
            $this->view->assign('managementUser', $this->request->getArgument('managementUser'));
        } else {
            $this->view->assign('managementUser', '');
        }

        // Vorhandene Wochentage abfuellen
        $days = [ 'Sonntag', 'Montag', 'Dienstag', 'Mittwoch', 'Donnerstag', 'Freitag', 'Samstag', 'Sonntag'];
        $weekdayHeaders = [];
        foreach (array_keys($eventsPerDayAndTime) as $dayNr) {
            $weekdayHeaders[] = [
                'nr' =>  $dayNr,
                'text' => $days[$dayNr]
            ];
        }

        /*
          echo($dayNr .'<br />');

          sort($eventsPerDayAndTime);
          exit(0);
        */
        $this->view->assign('weekdayHeaders', $weekdayHeaders);

        $this->view->assign('events', $eventsPerDayAndTime);
        $this->view->assign('message', $message);

        return $this->htmlResponse();
    }

    /**
     * List all events with possibility to sort and filter them.
     * This action is for organizers, so all the edit, create and delete links are visible here.
     *
     * @return void
     */
    public function organizerListAction(): ResponseInterface
    {
        $this->eventRepository->setShowOnlyActiveEntries(false);
        $events = $this->eventRepository->findBySearchAndSorting($this->getSearch(), $this->getSorting());
        $eventgroups = $this->eventgroupRepository->findCurrent();
        $eventgroupOptions=[];
        foreach ($eventgroups as $eg) {
            if ($eg->getEventsCount()>0) {
                $eventgroupOptions[]=$eg;
            }//->getTitle();
        }
        $this->view->assign('events', $events);
        $this->view->assign('eventgroupOptions', $eventgroupOptions);
        $this->view->assign('eventgroups', $eventgroups);
        $this->view->assign('organizer', $this->feUserRepository->findByUid($GLOBALS['TSFE']->fe_user->user['uid']));

        return $this->htmlResponse();
    }

    /**
     * Shows the detail view of an event
     *
     * @param \Ig\IgRuckzuckevent\Domain\Model\Event $event
     * @return void
     */
    public function showAction(\Ig\IgRuckzuckevent\Domain\Model\Event $event): ResponseInterface
    {
        $eventsWithSameTitle = $this->eventRepository->findForShow($event);
        $dates = $GLOBALS['TSFE']->fe_user->getKey('ses', 'tx_igruckzuckevent_dates');
        $GLOBALS['TSFE']->fe_user->removeSessionData();

        if ($this->request->hasArgument('managementUser') && UserUtility::isAdminLoggedIn()) {
            $user = $this->feUserRepository->findByUid($this->request->getArgument('managementUser'));
            $this->view->assign('managementUser', $this->request->getArgument('managementUser'));
        } elseif ($GLOBALS['TSFE']->loginUser) {
            $user = $this->feUserRepository->findByUid($GLOBALS['TSFE']->fe_user->user['uid']);
        }

        if ($this->request->hasArgument('dateErrors')) {
            $this->view->assign('dateErrors', $this->request->getArgument('dateErrors'));
        }

        $subscribeLink = $this->uriBuilder->reset()
                       ->uriFor('subscribe', null, 'Registration');

        /*->setArguments([
          'tx_igruckzuckevent_events' => [
          'controller' => 'Registration',
          'action' => 'subscribe'
          ]
          ])
          ->setUseCacheHash(false)
          ->build();
        */


        $eventsByWeekdays = [];
        foreach ($eventsWithSameTitle as $eventWithSameTitle) {
            $day = $eventWithSameTitle->getIsoDay();
            $time = $eventWithSameTitle->getTimeFrom();

            if (!is_array($eventsByWeekdays[$day])) {
                $eventsByWeekdays[$day] = [];
            }
            if (!is_array($eventsByWeekdays[$day][$time])) {
                $eventsByWeekdays[$day][$time] = [];
            }
            if (!in_array($eventWithSameTitle, $eventsByWeekdays[$day][$time])) {
                $eventsByWeekdays[$day][$time][] = $eventWithSameTitle;
            }
        }

        $this->view->assign('event', $event);
        $this->view->assign('eventsByWeekdays', $eventsByWeekdays);
        $this->view->assign('dates', $dates);
        $this->view->assign('user', $user);
        $this->view->assign('subscribeLink', $subscribeLink);
        $this->view->assign('selectedSubscription', GeneralUtility::_POST('tx_igruckzuckevent_events')['subscription']);

        return $this->htmlResponse();
    }

    /**
     * Shows form to create a new event
     *
     * @return void
     */
    public function newAction(): ResponseInterface
    {
        $this->view->assign('organizer', $GLOBALS['TSFE']->fe_user->user);

        return $this->htmlResponse();
    }

    /**
     * Action called when the Form in the new-Action is submitted.
     * We validate the event with our EventFormValidator, handle the file uploads and finally persist the event in the database.
     *
     * @param \Ig\IgRuckzuckevent\Domain\Model\Event $newEvent
     * @param boolean $repeat
     * @param int $repeatMode
     * @param string $repeatTill
     * @return void
     * @TYPO3\CMS\Extbase\Annotation\Validate("Internetgalerie\IgDynval\Validation\Validator\DynamicValidator", param="newEvent")
     * @TYPO3\CMS\Extbase\Annotation\Validate("\Ig\IgRuckzuckevent\Domain\Validator\EventFormValidator", param="newEvent")
     */
    public function createAction(\Ig\IgRuckzuckevent\Domain\Model\Event $newEvent, $repeat = false, $repeatMode = 1, $repeatTill = ''): ResponseInterface
    {
        $this->handleUpload($newEvent);

        if ($repeatTill != '') {
            $repeatTill = new \DateTime($repeatTill);
        }

        $eventgroup = GeneralUtility::makeInstance(Eventgroup::class);
        $eventgroup->setFirstDate(clone $newEvent->getDateTimeFrom());
        $eventgroup->setRepeated($repeat);

        if ($repeat && $repeatTill != '') {
            $eventgroup->setRepeatTill($repeatTill);
            $repeatTill->modify('+1 day');
        } elseif ($repeat && $repeatTill == '') {
            $repeatTill = clone $newEvent->getDateTimeFrom();
            $repeatTill->modify('+5 years');
        }

        if ($repeat) {
            $period = new \DatePeriod($newEvent->getDateFrom(), new \DateInterval('P' . $repeatMode . 'W'), $repeatTill);

            $i = 0; // We only use the period for the loop, the actual dates are calculated by the events date + $i weeks
            foreach ($period as $date) {
                $eventInGroup = clone $newEvent;
                $dateFrom = clone $eventInGroup->getDateFrom();

                $dateFrom->modify('+' . ($i++ * $repeatMode) . ' week');
                $eventInGroup->setDateFrom($dateFrom);
                $eventInGroup->setEventgroup($eventgroup);

                $this->eventRepository->add($eventInGroup);

                //$eventgroup->addEvent($eventInGroup);
            }
        } else {
            $this->eventRepository->add($newEvent);

            $newEvent->setEventgroup($eventgroup);
            //$eventgroup->addEvent($newEvent);
        }
        $this->eventgroupRepository->add($eventgroup);

        return $this->redirect('organizerList');
    }

    /**
     * Shows the edit form of an event
     *
     * @param \Ig\IgRuckzuckevent\Domain\Model\Event $event
     * @return void
     * @TYPO3\CMS\Extbase\Annotation\IgnoreValidation $event
     */
    public function editAction(\Ig\IgRuckzuckevent\Domain\Model\Event $event): ResponseInterface
    {
        $this->view->assign('event', $event);

        return $this->htmlResponse();
    }

    /**
     * This action is called when the edit form of an event is submitted.
     * It persists the changes made to the event to the database.
     * The event is validated by our EventFormValidator.
     *
     * @param \Ig\IgRuckzuckevent\Domain\Model\Event $event
     * @param int $saveMode
     * @return void
     * @TYPO3\CMS\Extbase\Annotation\Validate("Internetgalerie\IgDynval\Validation\Validator\DynamicValidator", param="event")
     * @TYPO3\CMS\Extbase\Annotation\Validate("\Ig\IgRuckzuckevent\Domain\Validator\EventFormValidator", param="event")
     */
    public function updateAction(\Ig\IgRuckzuckevent\Domain\Model\Event $event, $saveMode = 1): ResponseInterface
    {
        $arguments = $this->request->getArguments();
        $eventgroup = $event->getEventgroup();

        if ($saveMode == 3) { // Save changes for single event and put it in new eventgroup
            // Remove images to delete
            if (isset($arguments['imagesToDelete']) && !empty($arguments['imagesToDelete'])) {
                $imagesToDelete = [];
                foreach ($event->getImages() as $image) {
                    if (in_array($image->getUid(), $arguments['imagesToDelete'])) {
                        $imagesToDelete[] = $image;
                    }
                }
                foreach ($imagesToDelete as $imageToDelete) {
                    $event->removeImage($imageToDelete);
                }
            }
            $this->handleUpload($event);

            if ($eventgroup->getEvents()->count() > 1) {
                $newEventgroup = GeneralUtility::makeInstance(Eventgroup::class);
                $newEventgroup->setFirstDate(clone $event->getDateTimeFrom());
                $newEventgroup->setRepeated(false);

                $event->setEventgroup($newEventgroup);
                //$newEventgroup->addEvent($event);
                $this->eventgroupRepository->add($newEventgroup);
            }

            $this->eventRepository->update($event);
        } elseif ($saveMode == 2) { // Save changes for the following events and put them in new eventgroup
            $newEventgroup = GeneralUtility::makeInstance(Eventgroup::class);
            $newEventgroup->setFirstDate(clone $eventgroup->getFirstDate());
            $newEventgroup->setRepeated($eventgroup->getRepeated());
            $events = [];

            foreach ($eventgroup->getEvents() as $eventInGroup) {
                if ($eventInGroup->getDateTimeFrom() >= $event->getDateTimeFrom()) {
                    $eventInGroup->setActive($event->getActive());
		    $eventInGroup->setTitle($event->getTitle());
		    $eventInGroup->setSubtitle($event->getSubtitle());
                    $eventInGroup->setDescription($event->getDescription());
                    $eventInGroup->setTimeFrom($event->getTimeFrom());
                    $eventInGroup->setTimeTo($event->getTimeTo());
                    $eventInGroup->setMaxRegistrations($event->getMaxRegistrations());
                    $eventInGroup->setEventgroup($newEventgroup);
                    $this->eventRepository->update($eventInGroup);
                    $events[] = $eventInGroup;
                }
            }

            /*foreach($events as $eventInGroup) {
              $newEventgroup->addEvent($eventInGroup);
              $eventgroup->removeEvent($eventInGroup);
              $this->eventRepository->update($eventInGroup);
              }*/

            $this->eventgroupRepository->add($newEventgroup);
            $this->eventgroupRepository->update($eventgroup);
        } else { // Save changes for all the events in eventgroup
            foreach ($eventgroup->getEvents() as $eventInGroup) {
                $eventInGroup->setActive($event->getActive());
		$eventInGroup->setTitle($event->getTitle());
		$eventInGroup->setSubtitle($event->getSubtitle());
                $eventInGroup->setDescription($event->getDescription());
                $eventInGroup->setTimeFrom($event->getTimeFrom());
		$eventInGroup->setTimeTo($event->getTimeTo());
		$eventInGroup->setMaxRegistrations($event->getMaxRegistrations());

                $this->eventRepository->update($eventInGroup);
            }
        }

        return $this->redirect('organizerList');
    }

    /**
     * Deletes an event.
     *
     * @param \Ig\IgRuckzuckevent\Domain\Model\Event $event
     * @param string $deleteMode
     * @return void
     */
    public function deleteAction(\Ig\IgRuckzuckevent\Domain\Model\Event $event, $deleteMode = null): ResponseInterface
    {
        switch ($deleteMode) {
        case 'all':
            $eventgroup = $event->getEventgroup();
            foreach ($eventgroup->getEvents() as $eventInGroup) {
                if ($eventInGroup->getRegistrations()->count() == 0) {
                    $this->eventRepository->remove($eventInGroup);
                }
            }
            return $this->redirect('organizerList');
            break;
        case 'following':
            $eventgroup = $event->getEventgroup();
            foreach ($eventgroup->getEvents() as $eventInGroup) {
                if ($eventInGroup->getRegistrations()->count() == 0 && $eventInGroup->getDateTimeFrom() >= $event->getDateTimeFrom()) {
                    $this->eventRepository->remove($eventInGroup);
                }
            }
            return $this->redirect('organizerList');
            break;
        case 'single':
            if ($event->getRegistrations()->count() == 0) {
                $this->eventRepository->remove($event);
            }
            return $this->redirect('organizerList');
            break;
        default:
            $this->view->assign('event', $event);
            break;
        }

        return $this->htmlResponse();
    }

    /**
     * Exports the reservations of an event to an XLSX-File openable in Microsoft Excel
     *
     * @param \Ig\IgRuckzuckevent\Domain\Model\Event $event
     * @return void
     */
    public function exportAction(\Ig\IgRuckzuckevent\Domain\Model\Event $event): ResponseInterface
    {
        $phpExcelService = GeneralUtility::makeInstanceService('phpexcel');
        $phpExcel = $phpExcelService->getPHPExcel();
        $phpExcel->setActiveSheetIndex(0);

        $rowNr=1;
        $colNr=0;
        // First row: Output event title and registration count
        $phpExcel->getActiveSheet()->setCellValueByColumnAndRow($colNr++, $rowNr, $event->getTitle() .': '.$event->getDateFrom()->format('d.m.Y') .' '.$event->getTimeFromText());

        // Output registration count
        $phpExcel->getActiveSheet()->setCellValueByColumnAndRow($colNr++, $rowNr, 'Anzahl Anmeldungen: ' . $event->getRegistrations()->count());

        // Set first row bold
        $phpExcel->getActiveSheet()->getStyle('A1:C1')->getFont()->setBold(true);


        // Leerzeile
        $rowNr++;

        // Zeile mit Überschriften
        //$phpExcel->getActiveSheet()->SetCellValue('A2', 'Anrede');
        $rowNr++;
        $colNr=0;
        $phpExcel->getActiveSheet()->SetCellValueByColumnAndRow($colNr++, $rowNr, 'Name');
        $phpExcel->getActiveSheet()->SetCellValueByColumnAndRow($colNr++, $rowNr, 'Vorname');
        $phpExcel->getActiveSheet()->SetCellValueByColumnAndRow($colNr++, $rowNr, 'E-Mail-Adresse');
        $phpExcel->getActiveSheet()->SetCellValueByColumnAndRow($colNr++, $rowNr, 'Telefon');
        $phpExcel->getActiveSheet()->SetCellValueByColumnAndRow($colNr++, $rowNr, 'Gebucht am');

        // Set second row bold
        $phpExcel->getActiveSheet()->getStyle('A'.$rowNr.':E'.$rowNr)->getFont()->setBold(true);


        // For each registration of the event set the values of the attributes to the according column
        foreach ($event->getRegistrations() as $registration) {
            $rowNr++;
            $colNr=0;
            $feUser=$registration->getFeUser();
            //            $phpExcel->getActiveSheet()->SetCellValue('A' . $i, $feUser->getGender() == 'm' ? 'Herr' : 'Frau');
            $phpExcel->getActiveSheet()->SetCellValueByColumnAndRow($colNr++, $rowNr, $feUser->getLastName());
            $phpExcel->getActiveSheet()->SetCellValueByColumnAndRow($colNr++, $rowNr, $feUser->getFirstName());
            $phpExcel->getActiveSheet()->SetCellValueByColumnAndRow($colNr++, $rowNr, $feUser->getUsername());
            //setCellValueExplicitByColumnAndRow
            $phpExcel->getActiveSheet()->SetCellValueExplicitByColumnAndRow($colNr++, $rowNr, $feUser->getTelephone(), \PHPExcel_Cell_DataType::TYPE_STRING);
            $phpExcel->getActiveSheet()->SetCellValueByColumnAndRow($colNr++, $rowNr, $registration->getCrdate()->format('d.m.Y H:i'));
        }

        // Set column width automatically according to their content
        foreach (range('A', 'E') as $column) {
            $phpExcel->getActiveSheet()->getColumnDimension($column)->setAutoSize(true);
        }

        // Temporary filename
        $fileName = 'typo3temp/event-' . $event->getUid() . '-' . time() . '.xlsx';

        // Create an Excel Writer
        $excelWriter = $phpExcelService->getInstanceOf('PHPExcel_Writer_Excel2007', $phpExcel);
        // Save to temporary file
        $excelWriter->save($fileName);

        // Set headers
        header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet');
        header('Content-Disposition: attachment;filename="' . str_replace('typo3temp/', '', $fileName) . '"');
        header('Cache-Control: max-age=0');
        // If you're serving to IE 9, then the following may be needed
        header('Cache-Control: max-age=1');
        // If you're serving to IE over SSL, then the following may be needed
        header('Expires: Mon, 26 Jul 1997 05:00:00 GMT');
        // Date in the past
        header('Last-Modified: ' . gmdate('D, d M Y H:i:s') . ' GMT');
        // always modified
        header('Cache-Control: cache, must-revalidate');
        // HTTP/1.1
        header('Pragma: public');
        // HTTP/1.0
        header('Content-Length: ' . filesize($fileName));
        //header('Pragma: no-cache');

        echo file_get_contents($fileName);
        // Output file

        unlink($fileName);
        // Delete temporary file

        exit();

        return $this->htmlResponse();
    }

    /**
     * Get the search array
     *
     * @return array
     */
    private function getSearch()
    {
        if ($this->request) {
            $arguments = $this->request->getArguments();
        } else {
            $arguments = [];
        }

        if (isset($arguments['search'])) {
            $search = $arguments['search'];
        } else {
            $search = [];
        }

        $this->view->assign('search', $search);

        return $search;
    }

    /**
     * Get the sorting array for the repository
     *
     * @return array
     */
    private function getSorting()
    {
        if ($this->request) {
            $arguments = $this->request->getArguments();
        } else {
            $arguments = [];
        }

        if (isset($arguments['sorting'])) {
            $sorting = $arguments['sorting'];
        } else {
            // If no sorting was requested, use default sorting
            $sorting = [];
            $sorting['field'] = 'dateFrom';
            $sorting['direction']['dateFrom'] = 'ASC';
            $sorting['direction']['title'] = 'ASC';
        }

        // This is the sorting array given to the repository
        $sortingArr = [$sorting['field'] => $sorting['direction'][$sorting['field']]];
        if ($sorting['field'] == 'dateFrom') {
            $sortingArr['timeFrom'] = $sorting['direction'][$sorting['field']];
        }

        // If the current sorting direction of the current sorting field is ASC, switch it  to DESC, but all others stay at ASC
        // Else switch all sorting directions to ASC
        if ($sorting['direction'][$sorting['field']] == 'ASC') {
            foreach ($sorting['direction'] as &$sortingDirection) {
                $sortingDirection = 'ASC';
            }

            $sorting['direction'][$sorting['field']] = 'DESC';
        } else {
            foreach ($sorting['direction'] as &$sortingDirection) {
                $sortingDirection = 'ASC';
            }
        }

        $this->view->assign('sorting', $sorting);

        return $sortingArr;
    }

}
